{******************************************************************************

 ANALOG DEVICES 
 EUROPEAN DSP APPLICATIONS
 
 versatile FLASH utility for embedded systems
  
 Motorola S2 BNM parser,					   Ver: 0.90.0
 please use spl21 yourexe yourprom -loader -2181 -s2
 to generate your BNM. Otherwise this parser won't work.
 
	created     25-SEP-97 by hs
	last change 25-SEP-97 by hs

 Status:
 	verfied for ADmsp402/403 EB

 ******************************************************************************
}
.module	mot_s2_reader;

.include	<constant.h>;

.var 	b_p_l;				{ bytes per line }
.var	checksum;			{ line checksum }
.var	t_byte;				{ total number of bytes }
.var	offset;				{ addtl. offset address }
.global offset;
.var	mflag;				{ for monitor update }
.global	mflag;
.init	mflag: 0x0;

.var	bootcode[96];			{ holds modified boot loader }
.init	bootcode: <bc.cod>;		{ necessary for all files }
.global bootcode;

.var	mot_rev[5];
.global mot_rev;
.init	mot_rev: 0x302E, 0x3930, 0x2E30, 0x0D0A, 0x0000;

.entry mot_read;

mot_read:
	ar = 0x0;
	dm(t_byte) = ar;
	
	i0 = ^bootcode;			{ pointer to monitor bootldr }
	m0 = 1;
	l0 = 0;

	call get_char_ax1;		{ wait for start of line }
	ay0 = 0x1b;			{ '<ESC>' }
	ar = ax1 - ay0;
	if eq rts;			{ return to monitor }
	
	ay0 = 0x53;			{ 'S' }
	ar = ax1 - ay0;
	if ne jump mot_read;

	call get_char_ax1_to;		{ wait for second char of line }
	ay0 = 0x32;			{ '2' 0x96 }
	ar = ax1 - ay0;
	if ne jump mot_read;

mot_reent:
	call ascii_8bit;		{ get number of bytes in line }
	ay0 = 4;
	ar = dm(rbit_out);
	ar = ar - ay0;
	dm(b_p_l) = ar;
	
	call ascii_8bit;		{ get high 8 bit address offset }
	ay0 = dm(offset);
	ar = dm(rbit_out);
	ar = ar + ay0;
	dm(addr_hi) = ar;

	call ascii_16bit;		{ get low 16 bit address offset }
	ar = dm(rbit_out);
	dm(addr_lo) = ar;
		
	ar = dm(b_p_l);
	cntr = ar;
	do rec_lp until ce;
		
		call ascii_8bit;	{ get value }

		ar = dm(t_byte);
		ar = ar + 1;
		dm(t_byte)=ar;

		ay0 = 96;
		af = ar - ay0;
		if gt jump r_lp1;
		
		ay0 = dm(i0,m0);
		dm(d_byte) = ay0;	{ write value from mon bootldr }
		jump r_lp11;

	r_lp1:	ax1 = dm(mflag);	{ test for mon update }
		af = pass ax1;
		if eq jump r_lp9;	{ no - copy data }
	
		ar = dm(t_byte);
		ay1 = 109;		{ byte number to change }
		af = ar - ay1;
		if eq jump r_lp2;
		jump r_lp3;		{ transfer from file }
	r_lp2:	ar = 0x41;		{ new value }
		jump r_lp10;		{ go on writing }
		
	r_lp3:	ar = dm(t_byte);
		ay1 = 110;		{ byte number to change }
		af = ar - ay1;
		if eq jump r_lp4;
		jump r_lp5;		{ transfer from file }
	r_lp4:	ar = 0xC0;		{ new value }
		jump r_lp10;		{ go on writing }
		
	r_lp5:	ar = dm(t_byte);
		ay1 = 325;		{ byte number to change }
		af = ar - ay1;
		if eq jump r_lp6;
		jump r_lp7;
	r_lp6:	ar = 0x41;
		jump r_lp10;		{ go on writing }

	r_lp7:	ar = dm(t_byte);
		ay1 = 326;		{ byte number to change }
		af = ar - ay1;
		if eq jump r_lp8;
		jump r_lp9;
	r_lp8:	ar = 0xC0;		{ new value }
		call ch_flag;		{ reset flag }
		jump r_lp10;		{ go on writing }
		
	r_lp9:	ar = dm(rbit_out);
	r_lp10:	dm(d_byte) = ar;	{ write value }
		
	r_lp11:	call prog_byte;		{ write the byte out to FLASH }
		call auto_inc;
		ax1 = 0x2e;
		call out_char_ax1;

	rec_lp:	nop;

	call ascii_8bit;		{ get checksum }
	
	ay0 = dm(rbit_out);
	ar = dm(checksum);
	ar = ar + ay0;
	dm(checksum) = ar;

mot_restart:

	call get_char_ax1_to;
        none = pass ax1;
        if lt rts;			{ no character, timed out, exit screen }
        
	ay0 = 0x53;			{ wait for next line start "S" }
	ar = ax1 - ay0;
	if ne jump mot_restart;
	
	sr0 = ax1;
	sr = lshift sr0 by 8 (lo);
	
	call get_char_ax1_to;
	sr1 = ax1;
	
	sr = sr or lshift sr1 by 0 (lo);
	
	ay0 = 0x5332;			{ wait for next line start "S2" }
	ar = sr0 - ay0;			{ new data line ? }
	if eq jump mot_reent;		{ yes - next line }

	ay0 = 0x5339;			{ wait for next line start "S9" }
	ar = sr0 - ay0;			{ end of file ? }
	if eq rts;			{ yes - return }
	
mot_error:
	rts;
	
ch_flag:
	ay0 = 0x0;			{ reset flag to save time }
	dm(mflag) = ay0;
	rts;

.endmod;